@@ -52,7 +52,7 @@ use Net::Silk qw( :basic );
 use Net::Silk::IPAddr;
 use Net::Silk::IPSet;
 
-use Scalar::Util qw( looks_like_number );
+use Scalar::Util qw( looks_like_number refaddr );
 
 use overload (
 #  '|'    => \&_or,
@@ -84,10 +84,10 @@ use overload (
   'eq'   => \&_eq,
   'ne'   => \&_ne,
   #'0+'   => \&cardinality,
-  '""'   => \&_str,
-#  'bool' => \&_bool,
+  'bool' => \&cardinality,
   '<>'   => \&_fh_iter,
   '%{}'  => \&_me_hash,
+  '""'   => sub { refaddr shift },
 );
 
 use constant SILK_VIRTUAL_BAG_CLASS => 'Net::Silk::VirtualBag';
@@ -342,7 +342,7 @@ sub _autoconvert {
   $self->_autoconvert_enabled;
 }
 
-sub ipset {
+sub as_ipset {
   my $self = shift;
   croak("invalid bag key type for ipset") unless $self->_is_ipaddr;
   my $s = SILK_IPSET_CLASS->new;
@@ -435,6 +435,21 @@ sub contains {
   $self->get($_[0]) ? 1 : 0;
 }
 
+sub cardinality {
+  my $self = shift;
+  my $card;
+  eval { $card = $self->_cardinality };
+  if ($@) {
+    if ($@ =~ /overflow/) {
+      die "bag cardinality overflow (too many ipv6 keys)";
+    }
+    else {
+      croak $@;
+    }
+  }
+  $card;
+}
+
 sub _fh_iter {
   my $self = shift;
   my $attr = $Attr{$self} ||= _attr_init($self);
@@ -469,6 +484,9 @@ sub iter {
       $iter = undef;
       return @items;
     }
+    # keep $self in scope to prevent premature destruction
+    no warnings;
+    $self;
     my @res = Net::Silk::Bag::iter_xs::next($iter, $key_class, $cnt_class);
     if (!@res) {
       $iter = undef;
@@ -847,14 +865,14 @@ sub add {
   while (@_) {
     my $map = shift;
     if (my $vbag = _bagify($map)) {
-      my($iter, $sz);
+      my $iter;
       if (UNIVERSAL::isa($map, SILK_BAG_CLASS)) {
         #print STDERR "BAG ADD\n";
         Net::Silk::Bag::_add_bag($self, $map);
         next;
       }
       #print STDERR "VBAG ADD\n";
-      if ($self->cardinality > $sz) {
+      if ($self->cardinality > $vbag->cardinality) {
         $iter = $vbag->iter_bag;
         while ($_ = $iter->()) {
           $self->incr(@$_);
@@ -890,11 +908,10 @@ sub remove {
   while (@_) {
     my $map = shift;
     if (my $vbag = _bagify($map)) {
-      my($sz, $iter);
+      my $iter;
       $self->_merge_to_key_type($map)
         if UNIVERSAL::isa($map, SILK_BAG_CLASS);
-      $sz = $vbag->cardinality;
-      if ($self->cardinality > $sz) {
+      if ($self->cardinality > $vbag->cardinality) {
         $iter = $vbag->iter_bag;
         while ($_ = $iter->()) {
           $self->decr(@$_);
@@ -957,8 +974,6 @@ sub _ne {
 
 ###
 
-sub _str { overload::StrVal($_[0]) }
-
 sub _me_hash {
   my $self = shift;
   my $attr = $Attr{$self} ||= _attr_init($self);
@@ -1391,7 +1406,7 @@ Return the minimum value contained in the bag.
 
 =over
 
-=item ipset()
+=item as_ipset()
 
 Return a L<Net::Silk::IPSet> containing the keys of the bag
 in cases where the keys are IP addresses.
@@ -1470,11 +1485,11 @@ The following operators are overloaded and work with bag objects:
 
 =head1 SEE ALSO
 
-L<Net::Silk>, L<Net::Silk::RWRec>, L<Net::Silk::IPSet>, L<Net::Silk::IPWildcard>, L<Net::Silk::Pmap>, L<Net::Silk::IPAddr>, L<Net::Silk::TCPFlags>, L<Net::Silk::ProtoPort>, L<Net::Silk::File>, L<Net::Silk::Site>, L<silk(7)>
+L<Net::Silk>, L<Net::Silk::RWRec>, L<Net::Silk::IPSet>, L<Net::Silk::Pmap>, L<Net::Silk::IPWildcard>, L<Net::Silk::Range>, L<Net::Silk::CIDR>, L<Net::Silk::IPAddr>, L<Net::Silk::TCPFlags>, L<Net::Silk::ProtoPort>, L<Net::Silk::File>, L<Net::Silk::Site>, L<silk(7)>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright (C) 2011-2015 by Carnegie Mellon University
+Copyright (C) 2011-2016 by Carnegie Mellon University
 
 Use of the Net-Silk library and related source code is subject to the
 terms of the following licenses:
